//==============================================================================
// Product name: DBDiskBenchmark
// Copyright 2003-2004 AidAim Software LLC.
// Web site: http://www.aidaim.com
//==============================================================================
unit MainUnit;

{DEFINE NO_PROGRESS}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, DB, Spin, ACRMain, Grids,
  FormProgress,  TinyDB
;

type
  TForm1 = class(TForm)
    bnTestACRTable: TButton;
    Label4: TLabel;
    seRecordCount: TSpinEdit;
    bnTestAll: TButton;
    Button5: TButton;
    GroupBox1: TGroupBox;
    sgResults: TStringGrid;
    ACRTable1: TACRTable;
    ACRDatabase1: TACRDatabase;
    TinyTable1: TTinyTable;
    TinyDatabase1: TTinyDatabase;
    bnTinyDB: TButton;
    procedure Button5Click(Sender: TObject);
    procedure bnTestACRTableClick(Sender: TObject);
    procedure bnTestAllClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure bnTinyDBClick(Sender: TObject);
  private
    FTime:  Cardinal;
    FTime1: Cardinal;
  private
    { Private declarations }
    procedure ShowProgress(
                            Dataset:    TDataset;
                            Msg:        String;
                            TestNo:     Integer;
                            UseIndexes: Boolean
                         );
    procedure WriteToLog(
                            Dataset:    TDataset;
                            Msg:        String;
                            TestNo:     Integer;
                            Time:       Cardinal;
                            UseIndexes: Boolean
                         );
    procedure CreateTable(Dataset: TDataset; UseIndexes: Boolean);
    procedure SetActiveIndex(Dataset: TDataset; IndexName: String);
    procedure StartTransaction(Dataset: TDataset);
    procedure Commit(Dataset: TDataset);
    procedure InsertTest(Dataset: TDataset; UseIndexes: Boolean);
    procedure NavigateByNextTest(Dataset: TDataset; UseIndexes: Boolean);
    procedure NavigateByNextFilteredTest(Dataset: TDataset; UseIndexes: Boolean);
    procedure EditTest(Dataset: TDataset; UseIndexes: Boolean);
    procedure RecNoTest(Dataset: TDataset; UseIndexes: Boolean);
    procedure RecNoFilteredTest(Dataset: TDataset; UseIndexes: Boolean);
    procedure LocateByFIntegerTest(Dataset: TDataset; UseIndexes: Boolean);
    procedure LocateByFStringTest(Dataset: TDataset; UseIndexes: Boolean);
    procedure DeleteTest(Dataset: TDataset; UseIndexes: Boolean);

    procedure TestAll(Dataset: TDataset; UseIndexes: Boolean);
  public
    { Public declarations }
  end;

const
  tests = 9;
  logFileName = 'test.csv';
var
  Form1: TForm1;
  F: text;

const NO_TIME = $FFFFFFFF;

procedure WriteToFile(Str: String);

implementation

{$R *.dfm}

function GenerateString(
                       Len : Integer // serial length
                       ) : String; // returns serial
var i,x : integer;
    s : string;
    c : char;
begin
 s := '';

 for i := 1 to len do
  begin
   x := Random(101);
   if ((x mod 2) =  0) then
    c := chr(65+(Random(260000000) mod 26))
   else
    c := chr(48+(Random(100000000) mod 10));

   s := s + c;
  end; //len
 result := s;
end; // GenerateString


procedure TForm1.ShowProgress(
                            Dataset:    TDataset;
                            Msg:        String;
                            TestNo:     Integer;
                            UseIndexes: Boolean
                         );
var EngineName: String;
    f:          Double;
begin
{$IFDEF NO_PROGRESS}
Exit;
{$ENDIF}
   if (UseIndexes) then
    Msg := Msg + ' with indexes'
   else
    Msg := Msg + ' without indexes';
 if (Dataset = ACRTable1) then
  EngineName := sgResults.Cells[1,0];
 if (Dataset = TinyTable1) then
  EngineName := sgResults.Cells[2,0];
 if (UseIndexes) then
  TestNo := TestNo+tests;
 f := TestNo-1;
 fmProgress.Init(Msg,EngineName,Round(f / 14.0 * 100.0));
end; // ShowProgress


procedure TForm1.WriteToLog(
                            Dataset:    TDataset;
                            Msg:        String;
                            TestNo:     Integer;
                            Time:       Cardinal;
                            UseIndexes: Boolean
                            );
var Column: Integer;
begin
   if (UseIndexes) then
    Msg := Msg + ' with indexes'
   else
    Msg := Msg + ' without indexes';
 if (Dataset = ACRTable1) then
  Column := 1;
 if (Dataset = TinyTable1) then
  Column := 2;
 if (UseIndexes) then
  Inc(TestNo,tests);
 if (Time = NO_TIME) then
  begin
   sgResults.Cells[0,TestNo] := Msg + ' ...';
   sgResults.Cells[Column,TestNo] := '';
  end
 else
  begin
   sgResults.Cells[0,TestNo] := Msg;
   sgResults.Cells[Column,TestNo] := IntToStr(Time) + ' ms';
   if TestNo=1 then
    WriteToFile('Products: ,'+sgResults.Cells[1,0]+','+sgResults.Cells[2,0]+',RecordCount: ,'+IntToStr(seRecordCount.Value));
   if (Column = 1) then
    WriteToFile(Msg+','+IntToStr(Time))
   else
    WriteToFile(Msg+',,'+IntToStr(Time));
  end;
 if (TestNo > tests * 2) then
  WriteToFile(#13#10);
 Application.ProcessMessages;
end;


procedure TForm1.CreateTable(Dataset: TDataset; UseIndexes: Boolean);
begin
 if (Dataset = ACRTable1) then
  begin
   ACRDatabase1.Close;
   if (ACRDatabase1.Exists) then
     ACRDatabase1.DeleteDatabase;
   ACRDatabase1.CreateDatabase;
   ACRDatabase1.Open;
   ACRTable1.Close;
   ACRTable1.FieldDefs.Clear;
   ACRTable1.FieldDefs.Add('ID',ftAutoInc);
   ACRTable1.FieldDefs.Add('FString',ftString,100);
   ACRTable1.FieldDefs.Add('FInteger',ftInteger,0);
   ACRTable1.IndexDefs.Clear;
   if (UseIndexes) then
    begin
     ACRTable1.IndexDefs.Add('index1','ID',[]);
     ACRTable1.IndexDefs.Add('index2','FString',[]);
     ACRTable1.IndexDefs.Add('index3','FInteger',[]);
    end;
   ACRTable1.CreateTable;
   ACRTable1.Open;
  end
 else
  begin
   TinyDatabase1.Close;
   if (FileExists(TinyDatabase1.FileName)) then
    DeleteFile(TinyDatabase1.FileName);
   TinyDatabase1.CreateDatabase(TinyDatabase1.DatabaseName);
   TinyDatabase1.Open;
   TinyTable1.Close;
   TinyTable1.FieldDefs.Clear;
   TinyTable1.FieldDefs.Add('ID',ftAutoInc);
   TinyTable1.FieldDefs.Add('FString',ftString,100);
   TinyTable1.FieldDefs.Add('FInteger',ftInteger,0);
   TinyTable1.IndexDefs.Clear;
   if (UseIndexes) then
    begin
     TinyTable1.IndexDefs.Add('index1','ID',[]);
     TinyTable1.IndexDefs.Add('index2','FString',[]);
     TinyTable1.IndexDefs.Add('index3','FInteger',[]);
    end;
   TinyTable1.CreateTable;
   TinyTable1.Open;
 end;
end; // Create table


procedure TForm1.SetActiveIndex(Dataset: TDataset; IndexName: String);
begin
 if (Dataset = ACRTable1) then
  TACRTable(Dataset).IndexName := IndexName;
 if (Dataset = TinyTable1) then
  TTinyTable(Dataset).IndexName := IndexName;
end; // SetActiveIndex


procedure TForm1.StartTransaction(Dataset: TDataset);
begin
 if (Dataset = ACRTable1) then
  ACRDatabase1.StartTransaction;
end; // StartTransaction


procedure TForm1.Commit(Dataset: TDataset);
begin
 if (Dataset = ACRTable1) then
  ACRDatabase1.Commit(False);
end; // Commit


procedure TForm1.InsertTest(Dataset: TDataset; UseIndexes: Boolean);
var i,x,y: Integer;
    str:   String;
begin
 FTime := 0;
 WriteToLog(Dataset,'Insert',1,NO_TIME,UseIndexes);
 ShowProgress(Dataset,'Insert',1,UseIndexes);
 y := seRecordCount.Value div 100;
 for i := 1 to seRecordCount.Value do
  begin
   x := random (MAXInt) mod (seRecordCount.Value div 10);
   str := GenerateString(100);
   FTime1 := GetTickCount;
   if ((i mod 1000) = 1) then
    StartTransaction(Dataset);
   Dataset.Insert;
   Dataset.FieldByName('FInteger').AsInteger := x;
   Dataset.FieldByName('FString').AsString := str;
   Dataset.Post;
   if (i = seRecordCount.Value) or ((i mod 1000) = 0) then
    Commit(Dataset);
   FTime1 := GetTickCount - FTime1;
   Inc(FTime,FTime1);
{$IFNDEF NO_PROGRESS}
   if ((i mod y) = 0) then
    begin
     fmProgress.gCurrent.Progress:= Round((i / seRecordCount.Value )* 100.0);
     Application.ProcessMessages;
     if (FormProgress.CloseFlag) then Exit;
    end;
{$ENDIF}
  end;
 WriteToLog(Dataset,'Insert',1,FTime,UseIndexes);
end; // Insert


procedure TForm1.NavigateByNextTest(Dataset: TDataset; UseIndexes: Boolean);
var i,y,x:  Integer;
    str:    String;
begin
 FTime := 0;
 WriteToLog(Dataset,'Navigate By Next',2,NO_TIME,UseIndexes);
 ShowProgress(Dataset,'Navigate By Next',2,UseIndexes);
 y := seRecordCount.Value div 100;
 i := 0;
 if (UseIndexes) then
  SetActiveIndex(Dataset,'index2')
 else
  SetActiveIndex(Dataset,'');
 FTime1 := GetTickCount;
 Dataset.First;
 FTime1 := GetTickCount - FTime1;
 Inc(FTime,FTime1);
 while not Dataset.Eof do
  begin
   FTime1 := GetTickCount;
   x := Dataset.FieldByName('FInteger').AsInteger;
   str := Dataset.FieldByName('FString').AsString;
   Dataset.Next;
   FTime1 := GetTickCount - FTime1;
   Inc(FTime,FTime1);
   Inc(i);
{$IFNDEF NO_PROGRESS}
   if ((i mod y) = 0) then
    begin
     fmProgress.gCurrent.Progress:= Round((i / seRecordCount.Value )* 100.0);
     Application.ProcessMessages;
     if (FormProgress.CloseFlag) then Exit;
    end;
{$ENDIF}
  end;
 WriteToLog(Dataset,'Navigate by Next',2,FTime,UseIndexes);
 SetActiveIndex(Dataset,'');
end; // NavigateByNext


procedure TForm1.NavigateByNextFilteredTest(Dataset: TDataset; UseIndexes: Boolean);
var i,y,x:  Integer;
    str:    String;
begin
 FTime := 0;
 WriteToLog(Dataset,'Navigate By Next with filter',3,NO_TIME,UseIndexes);
 ShowProgress(Dataset,'Navigate By Next with filter',3,UseIndexes);
 y := seRecordCount.Value div 100;
 if (UseIndexes) then
  SetActiveIndex(Dataset,'index2')
 else
  SetActiveIndex(Dataset,'');
 Dataset.Filtered := False;
 Dataset.Filter := 'ID >= 0';
 Dataset.Filtered := True;
 FTime1 := GetTickCount;
 Dataset.First;
 FTime1 := GetTickCount - FTime1;
 Inc(FTime,FTime1);
 i := 0;
 while not Dataset.Eof do
  begin
   FTime1 := GetTickCount;
   x := Dataset.FieldByName('FInteger').AsInteger;
   str := Dataset.FieldByName('FString').AsString;
   Dataset.Next;
   FTime1 := GetTickCount - FTime1;
   Inc(FTime,FTime1);
   Inc(i);
{$IFNDEF NO_PROGRESS}
   if ((i mod y) = 0) then
    begin
     fmProgress.gCurrent.Progress:= Round((i / seRecordCount.Value )* 100.0);
     Application.ProcessMessages;
     if (FormProgress.CloseFlag) then Exit;
    end;
{$ENDIF}
  end;
 WriteToLog(Dataset,'Navigate by Next with filter',3,FTime,UseIndexes);
 SetActiveIndex(Dataset,'');
 Dataset.Filtered := False;
end; // NavigateByNext Filtered


procedure TForm1.EditTest(Dataset: TDataset; UseIndexes: Boolean);
var i,y,x:  Integer;
    str:    String;
begin
 FTime := 0;
 WriteToLog(Dataset,'Edit',4,NO_TIME,UseIndexes);
 ShowProgress(Dataset,'Edit',4,UseIndexes);
 y := seRecordCount.Value div 100;
 for i := 1 to seRecordCount.Value do
  begin
   Dataset.RecNo := i;
   x := random (MAXInt) mod (seRecordCount.Value div 10);
   str := GenerateString(100);
   FTime1 := GetTickCount;
   if ((i mod 1000) = 1) then
    StartTransaction(Dataset);
   Dataset.Edit;
   Dataset.FieldByName('FInteger').AsInteger := x;
   Dataset.FieldByName('FString').AsString := str;
   Dataset.Post;
   if (i = seRecordCount.Value) or ((i mod 1000) = 0) then
    Commit(Dataset);
   FTime1 := GetTickCount - FTime1;
   Inc(FTime,FTime1);
{$IFNDEF NO_PROGRESS}
   if ((i mod y) = 0) then
    begin
     fmProgress.gCurrent.Progress:= Round((i / seRecordCount.Value )* 100.0);
     Application.ProcessMessages;
     if (FormProgress.CloseFlag) then Exit;
    end;
{$ENDIF}
  end;
 WriteToLog(Dataset,'Edit',4,FTime,UseIndexes);
end; // Edit


procedure TForm1.RecNoTest(Dataset: TDataset; UseIndexes: Boolean);
var i,x,y,low,medium,big: Integer;
begin
 FTime := 0;
 WriteToLog(Dataset,'Navigate by RecNo',5,NO_TIME,UseIndexes);
 ShowProgress(Dataset,'Navigate by RecNo',5,UseIndexes);
 if (UseIndexes) then
  SetActiveIndex(Dataset,'index2')
 else
  SetActiveIndex(Dataset,'');
 y := 10;
 for i := 1 to 1000 do
  begin
   FTime1 := GetTickCount;
   Dataset.RecNo := i;
   FTime1 := GetTickCount - FTime1;
   Inc(FTime,FTime1);
{$IFNDEF NO_PROGRESS}
   if ((i mod y) = 0) then
    begin
     fmProgress.gCurrent.Progress:= Round((i / 1000 )* 100.0);
     Application.ProcessMessages;
     if (FormProgress.CloseFlag) then Exit;
    end;
{$ENDIF}
  end;
 WriteToLog(Dataset,'Navigate by RecNo',5,FTime,UseIndexes);
 SetActiveIndex(Dataset,'');
end; // RecNo


procedure TForm1.RecNoFilteredTest(Dataset: TDataset; UseIndexes: Boolean);
var i,x,y,low,medium,big: Integer;
begin
 FTime := 0;
 WriteToLog(Dataset,'Navigate by RecNo with filter',6,NO_TIME,UseIndexes);
 ShowProgress(Dataset,'Navigate by RecNo with filter',6,UseIndexes);
 y := 10;
 if (UseIndexes) then
  SetActiveIndex(Dataset,'index2')
 else
  SetActiveIndex(Dataset,'');
 Dataset.Filtered := False;
 Dataset.Filter := 'ID >= 0';
 Dataset.Filtered := True;
 for i := 1 to 1000 do
  begin
   FTime1 := GetTickCount;
   Dataset.RecNo := i;
   FTime1 := GetTickCount - FTime1;
   Inc(FTime,FTime1);
{$IFNDEF NO_PROGRESS}
   if ((i mod y) = 0) then
    begin
     fmProgress.gCurrent.Progress:= Round((i / 1000 )* 100.0);
     Application.ProcessMessages;
     if (FormProgress.CloseFlag) then Exit;
    end;
{$ENDIF}
  end;
 WriteToLog(Dataset,'Navigate by RecNo with filter',6,FTime,UseIndexes);
 SetActiveIndex(Dataset,'');
 Dataset.Filtered := False;
end; // RecNo with filter


procedure TForm1.LocateByFIntegerTest(Dataset: TDataset; UseIndexes: Boolean);
var i,x,y,OldRecNo: Integer;
begin
 FTime := 0;
 WriteToLog(Dataset,'Locate By FInteger',7,NO_TIME,UseIndexes);
 ShowProgress(Dataset,'Locate By FInteger',7,UseIndexes);
 y := 10;
 for i := 1 to 1000 do
  begin
   Dataset.RecNo := i;
   OldRecNo := Dataset.RecNo;
   x := Dataset.FieldByName('FInteger').AsInteger;
   Dataset.First;
   FTime1 := GetTickCount;
   if (not Dataset.Locate('FInteger',x,[])) then
     raise Exception.Create('Locate By FInteger test failed');
   FTime1 := GetTickCount - FTime1;
   Inc(FTime,FTime1);
   if (Dataset.FieldByName('FInteger').AsInteger <> x) then
     raise Exception.Create('Locate By FInteger test failed record found');
{$IFNDEF NO_PROGRESS}
   if ((i mod y) = 0) then
    begin
     fmProgress.gCurrent.Progress:= Round((i / 1000 )* 100.0);
     Application.ProcessMessages;
     if (FormProgress.CloseFlag) then Exit;
    end;
{$ENDIF}
  end;
 WriteToLog(Dataset,'Locate By FInteger',7,FTime,UseIndexes);
end; //


procedure TForm1.LocateByFStringTest(Dataset: TDataset; UseIndexes: Boolean);
var i,x,y,OldRecNo: Integer;
    s: string;
    t: Cardinal;
begin
 FTime := 0;
 WriteToLog(Dataset,'Locate By FString',8,NO_TIME,UseIndexes);
 ShowProgress(Dataset,'Locate By FString',8,UseIndexes);
 y := 10;
 for i := 1 to 1000 do
  begin
   Dataset.RecNo := i;
   OldRecNo := Dataset.RecNo;
   s := Dataset.FieldByName('FString').AsString;
   Dataset.First;
   FTime1 := GetTickCount;
   if (not Dataset.Locate('FString',s,[])) then
     raise Exception.Create('Locate By FString test failed');
   FTime1 := GetTickCount - FTime1;
   Inc(FTime,FTime1);
   if (Dataset.FieldByName('FString').AsString <> s) then
     raise Exception.Create('Locate By FString test failed record found');
{$IFNDEF NO_PROGRESS}
   if ((i mod y) = 0) then
    begin
     fmProgress.gCurrent.Progress:= Round((i / 1000 )* 100.0);
     Application.ProcessMessages;
     if (FormProgress.CloseFlag) then Exit;
    end;
{$ENDIF}
  end;
 WriteToLog(Dataset,'Locate By FString',8,FTime,UseIndexes);
end; //


procedure TForm1.DeleteTest(Dataset: TDataset; UseIndexes: Boolean);
var RecCount: Double;
    i,y:      Integer;
begin
 FTime := 0;
 WriteToLog(Dataset,'Delete',9,NO_TIME,UseIndexes);
 ShowProgress(Dataset,'Delete',9,UseIndexes);
 RecCount := dataset.RecordCount;
 i := 1;
 y := dataset.RecordCount div 100;
 while (dataset.RecordCount > 0) do
  begin
   FTime1 := GetTickCount;
   if ((i mod 1000) = 1) then
    StartTransaction(Dataset);
   dataset.Delete;
   if (dataset.RecordCount = 0) or ((i mod 1000) = 0) then
    Commit(Dataset);
   FTime1 := GetTickCount - FTime1;
   Inc(FTime,FTime1);
   Inc(i);
{$IFNDEF NO_PROGRESS}
   if ((i mod y) = 0) then
    begin
     fmProgress.gCurrent.Progress:= Round(((RecCount - dataset.recordCount) / RecCount)* 100.0);
     Application.ProcessMessages;
     if (FormProgress.CloseFlag) then Exit;
    end;
{$ENDIF}
  end;
 WriteToLog(Dataset,'Delete',9,FTime,UseIndexes);
end; // Delete


procedure TForm1.TestAll(Dataset: TDataset; UseIndexes: Boolean);
begin
 InsertTest(Dataset,UseIndexes);
 if (FormProgress.CloseFlag) then Exit;
 NavigateByNextTest(Dataset,UseIndexes);
 if (FormProgress.CloseFlag) then Exit;
 NavigateByNextFilteredTest(Dataset,UseIndexes);
 if (FormProgress.CloseFlag) then Exit;
 EditTest(Dataset,UseIndexes);
 if (FormProgress.CloseFlag) then Exit;
 RecNoTest(Dataset,UseIndexes);
 if (FormProgress.CloseFlag) then Exit;
 RecNoFilteredTest(Dataset,UseIndexes);
 if (FormProgress.CloseFlag) then Exit;
 LocateByFIntegerTest(Dataset,UseIndexes);
 if (FormProgress.CloseFlag) then Exit;
 LocateByFStringTest(Dataset,UseIndexes);
 if (FormProgress.CloseFlag) then Exit;
 DeleteTest(Dataset,UseIndexes);
 if (FormProgress.CloseFlag) then Exit;
end; //


procedure TForm1.Button5Click(Sender: TObject);
begin
 Application.Terminate;
 Application.ProcessMessages;
 Close;
end;

procedure TForm1.bnTestACRTableClick(Sender: TObject);
begin
 ACRDatabase1.Close;
 TinyDatabase1.Close;
 CreateTable(ACRTable1,False);
 TestAll(ACRTable1,False);
 if (FormProgress.CloseFlag) then
  begin
   fmProgress.Hide;
   Exit;
  end;
 CreateTable(ACRTable1,True);
 TestAll(ACRTable1,True);
 ACRDatabase1.Close;
 fmProgress.gOverall.Progress := 100;
 Application.ProcessMessages;
 fmProgress.Hide;
end;

procedure TForm1.bnTestAllClick(Sender: TObject);
begin
 bnTestACRTableClick(Sender);
 if (FormProgress.CloseFlag) then Exit;
 bnTinyDBClick(Sender);
end;


procedure TForm1.FormCreate(Sender: TObject);
begin
 sgResults.RowCount := tests * 2 + 1;
 sgResults.ColWidths[0] := 250;
 sgResults.Cells[0,0] := 'Test operation';
 sgResults.Cells[1,0] := 'Accuracer';
 sgResults.Cells[2,0] := 'TinyDB';
end;

procedure TForm1.bnTinyDBClick(Sender: TObject);
begin
 ACRDatabase1.Close;
 TinyDatabase1.Close;
 CreateTable(TinyTable1,False);
 TestAll(TinyTable1,False);
 if (FormProgress.CloseFlag) then
  begin
   fmProgress.Hide;
   Exit;
  end;
 CreateTable(TinyTable1,True);
 TestAll(TinyTable1,True);
 TinyDatabase1.Close;
 fmProgress.gOverall.Progress := 100;
 Application.ProcessMessages;
 fmProgress.Hide;
end;

procedure WriteToFile(Str: String);
var f: Text;
begin
 Assign(f,logFileName);
 if (FileExists(logFileName)) then
  Append(f)
 else
  ReWrite(f);
 Writeln(f,Str);
 Close(f);
end;

initialization

 Randomize;

end.
